def absolute():
    absolute = ['(\d+(,|\s|\.|-|\/|_)+\d+(,|\s|\.|-|\/|_)+\d+)', # 21/03/1997  21-03-1997  21.03.1997
                   '((monday|tuesday|wednesday|thursday|friday|saturday|sunday)(,|\s|\.|-|\/|_)\d+(,|\s|\.|-|\/|_)*\d*(,|\s|\.|-|\/|_)*\d*)', # Monday 21 3 
                ] 
    wod = ['(monday|tuesday|wednesday|friday|saturday|sunday)']
    wod_vn = ['(((thứ)|(thu)|(thuws))(\s+|\.+)*[2-8])|(((thứ)|(thu)|(thuws))(\s+|\.+)*((hai)|(bay)|(tư)|(tu)|(bốn)|(bon)|(năm)|(nam)|(lăm)|(lam)|(sáu)|(say)|(bảy)|(ba)))|((chủ nhật)|(chu nhat)|(cn))']

    day_vn = ['(((ngày)|(ngay)|(ngafy))(\s+|\.+)*(\d\d|\d)($|\s))|(((ngày)|(ngay)|(ngafy))\s+((không)|(khong)|(một)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi))(\s*((không)|(khong)|(một)|(mốt)|(mot)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi)))?(\s*((không)|(khong)|(một)|(mốt)|(mot)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi)))?)|((^|\s)\d\d?\s*((tháng)|(thang)))|((^|\s)((không)|(khong)|(một)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi))(\s*((không)|(khong)|(một)|(mốt)|(mot)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi)))?(\s*((không)|(khong)|(một)|(mốt)|(mot)|(hai)|(bay)|(bốn)|(bon)|(tư)|(tu)|(năm)|(lăm)|(nam)|(lam)|(sáu)|(sau)|(bảy)|(ba)|(tám)|(tam)|(chín)|(chin)|(mười)|(mươi)|(muoi)))?\s*((tháng)|(thang)))']

    month = ['(january|jan|february|feb|march|mar|april|apr|may|june|jun|july|jul|august|aug|september|sep|october|oct|november|nov|december|dec)']
    month_vn = ['(((tháng)|(thang)|(thasng))(\s+|\.+)*(\d\d|\d)($|\s))|(((tháng)|(thang))(\s+|\.+)*(một|mot|hai|ba|bốn|bon|tư|tu|năm|nam|lăm|lam|sáu|bảy|bay|tám|tam|chín|chin|muoi|mười một|muoi mot|mười hai|muoi hai|giêng|gieng|chạp|chap|mười))']

    year = ['('+'|'.join([str(i) for i in list(range(1900, 2050))])+')']

    only_number = ['^\d+$']

    short_abs = ['([0-3][0-9](,|\s|\.|-|\/|_)[0-1][0-9])|((^|\s)\d(,|\s|\.|-|\/|_)\d(\s|$))|([0-3][0-9](,|\s|\.|-|\/|_)[0-9](\s|$))|((^|\s)\d(,|\s|\.|-|\/|_)(1[0-2]))'] # 21 / 03

    return absolute, wod, wod_vn, day_vn, month, month_vn, year, only_number, short_abs

def relative():
    short_time = ['(((hôm)|(hom))\s*((qua)|(rồi)|(vừa rồi)|(roi)))|(((hôm)|(hom))\s*nay)|(((hôm)|(hom))\s*kia)|(((ngày)|(ngay))\s(((hôm)|(hom))\s)*((qua)|(rồi)|(vừa rồi)|(roi)))|(sáng\s(((hôm)|(hom))\s)*((qua)|(rồi)|(vừa rồi)|(roi)))|(trưa\s(((hôm)|(hom))\s)*((qua)|(rồi)|(vừa rồi)|(roi)))|(chiều\s(((hôm)|(hom))\s)*((qua)|(rồi)|(vừa rồi)|(roi)))|(tối\s(((hôm)|(hom))\s)*((qua)|(rồi)|(vừa rồi)|(roi)))|(((ngày)|(ngay))\s(((hôm)|(hom))\s)*nay)|(sáng\s(((hôm)|(hom))\s)*nay)|(trưa\s(((hôm)|(hom))\s)*nay)|(chiều\s(((hôm)|(hom))\s)*nay)|(tối\s(((hôm)|(hom))\s)*nay)|(((ngày)|(ngay))\s(((hôm)|(hom))\s)*mai)|(sáng\s(((hôm)|(hom))\s)*mai)|(trưa\s(((hôm)|(hom))\s)*mai)|(chiều\s(((hôm)|(hom))\s)*mai)|(tối\s(((hôm)|(hom))\s)*mai)|(((ngày)|(ngay))\s(((hôm)|(hom))\s)*mốt)|(sáng\s(((hôm)|(hom))\s)*mốt)|(trưa\s(((hôm)|(hom))\s)*mốt)|(chiều\s(((hôm)|(hom))\s)*mốt)|(tối\s(((hôm)|(hom))\s)*mốt)|(((ngày)|(ngay))\s(((hôm)|(hom))\s)*kia)|(sáng\s(((hôm)|(hom))\s)*kia)|(trưa\s(((hôm)|(hom))\s)*kia)|(chiều\s(((hôm)|(hom))\s)*kia)|(tối\s(((hôm)|(hom))\s)*kia)|(buổi\ssáng)|(buổi\strưa)|(buổi\schiều)|(buổi\stối)']

    long_time = ['(((tuần)|(tuan))\s((này)|(nay)))|(((tuần)|(tuan))\ssau)|(((tuần)|(tuan))\s((qua)|(rồi)|(roi)|(vừa rồi)))|(((tuần)|(tuan))\s((tới)|(toi)))|(((tháng)|(thang))\s((này)|(nay)))|(((tháng)|(thang))\ssau)|(((tháng)|(thang))\s((qua)|(rồi)|(roi)|(vừa rồi)))|(((tháng)|(thang))\s((tới)|(toi)))']
    
    adj_pattern_top = ['((đầu)|(dau))\s*((tuần)|(tuan)|(tháng)|(thang))']
    adj_pattern_middle = ['((giữa)|(giua))\s*((tuần)|(tuan)|(tháng)|(thang))']
    adj_pattern_bot = ['((cuối)|(cuoi))\s*((tuần)|(tuan)|(tháng)|(thang))']


    return short_time, long_time, adj_pattern_top, adj_pattern_middle, adj_pattern_bot


import torch.nn as nn
import torch.nn.functional as F


class FeedforwardLayer(nn.Module):
    ''' A two-feed-forward-layer module '''

    def __init__(self, d_in, d_hid, dropout=0.3):
        super(FeedforwardLayer,self).__init__()
        self.w_1 = nn.Linear(d_in, d_hid)  # position-wise
        self.w_2 = nn.Linear(d_hid, d_in)  # position-wise
        self.layer_norm = nn.LayerNorm(d_in, eps=1e-6)
        self.dropout = nn.Dropout(dropout)
        self.dropout2 = nn.Dropout(dropout)
    def forward(self, x):
        residual = x
        x = self.layer_norm(x)
        x = self.w_2(self.dropout(F.relu(self.w_1(x))))
        x = residual + self.dropout2(x)
        return x


import os
import pickle
import logging
import numpy as np
from typing import Any, Dict, List, Optional, Text, Tuple, Type

from rasa.nlu.components import Component
from rasa.nlu.classifiers.classifier import IntentClassifier
from rasa.nlu.config import RasaNLUModelConfig
from rasa.shared.nlu.training_data.training_data import TrainingData
from rasa.shared.nlu.training_data.message import Message

WORK_DIR = os.path.dirname(os.path.abspath(__file__))
MODEL_PATH = os.path.join(WORK_DIR, "weight/naive_bayes_model.pkl")

logger = logging.getLogger(__name__)

class NaiveBayesClassifier(IntentClassifier):
    name = "NaiveBayesClassifier"

    def __init__(self, component_config: Optional[Dict[Text, Any]] = None, learner = None) -> None:
        super(NaiveBayesClassifier, self).__init__(component_config)
        self.learner = learner

    def train(
        self,
        training_data: TrainingData,
        config: Optional[RasaNLUModelConfig] = None,
        **kwargs: Any,
    ) -> None:
        """Train this component.

        This is the components chance to train itself provided
        with the training data. The component can rely on
        any context attribute to be present, that gets created
        by a call to :meth:`components.Component.pipeline_init`
        of ANY component and
        on any context attributes created by a call to
        :meth:`components.Component.train`
        of components previous to this one."""
        pass

    def process(self, message: Message, **kwargs: Any) -> None:
        """Process an incoming message."""

        if not self.learner:
            intent = None
            intent_ranking = []
        else:
            text = message.data.get('text')
            output = self.predict(text)
            intent = output.get("intent")
            intent_ranking = output.get("intent_ranking")

        message.set("intent", intent, add_to_output=True)
        message.set("intent_ranking", intent_ranking, add_to_output=True)

    def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
        """Persist this component to disk for future loading."""

        pass

    @classmethod
    def load(
        cls,
        meta: Dict[Text, Any],
        model_dir: Text,
        model_metadata: Optional["Metadata"] = None,
        cached_component: Optional["Component"] = None,
        **kwargs: Any,
    ) -> "Component":
        """Load this component from file."""

        if not os.path.isfile(MODEL_PATH):
            logger.error(f"File not found. Cannot load Logistic Regression model: {MODEL_PATH}")
            return cls(component_config=meta)
        else:
            try:
                learner = pickle.load(open(MODEL_PATH, 'rb'))
                logger.debug(f"Load Logistic Regression model successfully ")
                return cls(meta, learner)
            except Exception as ex:
                logger.error(f"Cannot load Logistic Regression model: {MODEL_PATH}: error: {ex}")

    def predict(self, text: Text):
        text = [text]
        predict = self.learner.predict(text)
        confidence = np.max(self.learner.predict_proba(text)[0])
        return {
            "intent":{
                "name":predict[0],
                "confidence": confidence
            },
            "intent_ranking":[]
        }


import torch
# from model.layer import CharCNN, FeatureEmbedding
from vma_nlu.ner.pername_deeplearning.model.layer.charcnn import CharCNN
from vma_nlu.ner.pername_deeplearning.model.layer.featureEmbed import FeatureEmbedding

import torch
from torch import nn
from transformers import AutoModel

class WordRepresentation(nn.Module):
    def __init__(self, args):
        super().__init__()

        self.bert = AutoModel.from_pretrained(args.model_name_or_path)
        self.num_layer_bert = args.num_layer_bert

        self.use_char = args.use_char
        if self.use_char:
            self.char_feature = CharCNN(hidden_dim=args.char_hidden_dim,
                                        vocab_size=args.char_vocab_size, 
                                        embedding_dim=args.char_embedding_dim)

        self.use_pos = args.use_pos
        if self.use_pos:
            self.pos_feature = FeatureEmbedding(vocab_size=args.pos_vocab_size, 
                                                embedding_size=args.feature_embed_dim)

    def forward(self, input_ids, attention_mask, first_subword, char_ids, pos_ids):

        bert_output = self.bert(input_ids, attention_mask=attention_mask, output_hidden_states=True)
        bert_features = []
        for i in range(-1, -(self.num_layer_bert + 1), -1):
            bert_features.append(bert_output[2][i])
        bert_features = torch.cat(bert_features, dim=-1)

        bert_features = torch.cat([torch.index_select(bert_features[i], 0, first_subword[i]).unsqueeze(0) for i in range(bert_features.size(0))], dim=0)

        word_features = []
        if self.use_char:
            char_features = self.char_feature(char_ids)
            word_features.append(char_features)
        
        if self.use_pos:
            pos_features = self.pos_feature(pos_ids)
            word_features.append(pos_features)

        word_features = torch.cat(word_features, dim=-1)

        final_feature = torch.cat((bert_features, word_features), dim=-1)
        return final_feature


import os
import logging
from typing import Any, Dict, List, Optional, Text, Tuple, Type, Callable
from vma_nlu.ner.extractor import Extractor

from rasa.nlu.components import Component
from rasa.nlu.extractors.extractor import EntityExtractor
from rasa.nlu.config import RasaNLUModelConfig
from rasa.shared.nlu.training_data.training_data import TrainingData
from rasa.shared.nlu.training_data.message import Message
from vma_nlu.ner.pername_deeplearning.inference import Inference

logger = logging.getLogger(__name__)

class BiaffineExtractor(EntityExtractor):
    name = "BiaffineExtractor"

    def __init__(
        self,
        component_config: Optional[Dict[Text, Any]] = None,
        learner = None,
    ) -> None:
        super(BiaffineExtractor, self).__init__(component_config)
        self.learner = learner

    def train(
        self,
        training_data: TrainingData,
        config: Optional[RasaNLUModelConfig] = None,
        **kwargs: Any,
    ) -> None:
        """Train this component.

        This is the components chance to train itself provided
        with the training data. The component can rely on
        any context attribute to be present, that gets created
        by a call to :meth:`components.Component.pipeline_init`
        of ANY component and
        on any context attributes created by a call to
        :meth:`components.Component.train`
        of components previous to this one."""
        pass

    def process(self, message: Message, **kwargs: Any) -> None:
        """Process an incoming message."""
        text = message.data.get('text')
        if text:
            result = self.learner.inference(text)
            old_entities = message.data.get("entities")
            for e in result.get("entities"):
                if e.get("labels")[0].value == "PERSON":
                    e["extractor"] = "BiaffineExtractor"
                    e["value"] = e["value"].title()
                    old_entities.append(e)
            message.set("entities", old_entities, add_to_output=True)
        

    def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
        """Persist this component to disk for future loading."""

        label_set_path = self.component_config.get("label_set_path")
        char_vocab_path = self.component_config.get("char_vocab_path")
        checkpoint = self.component_config.get("checkpoint")

        return {"label_set_path": label_set_path, "char_vocab_path": char_vocab_path, "checkpoint": checkpoint}

    @classmethod
    def load(
        cls,
        meta: Dict[Text, Any],
        model_dir: Text,
        model_metadata: Optional["Metadata"] = None,
        cached_component: Optional["Component"] = None,
        **kwargs: Any,
    ) -> "Component":
        """Load this component from file."""
        label_set_path = meta.get("label_set_path")
        char_vocab_path = meta.get("char_vocab_path")
        checkpoint = meta.get("checkpoint")
        if not os.path.isfile(label_set_path) or not os.path.isfile(char_vocab_path) or not os.path.isfile(checkpoint):
            logger.error(f"File not found. Cannot load Biaffine Extractor model: {checkpoint}")
            return cls(component_config=meta)
        else:
            try:
                learner = Inference(label_set_path, char_vocab_path, checkpoint)
                logger.debug(f"Load Biaffine Extractor model successfully ")
                return cls(meta, learner)
            except Exception as ex:
                logger.error(f"Cannot load Biaffine Extractor model: {checkpoint}: error: {ex}")



import torch

def get_mask(max_seq_len, seq_len):
    mask = [[1]*seq_len[i]+[0]*(max_seq_len-seq_len[i]) for i in range(len(seq_len))]
    mask = torch.tensor(mask)
    mask = mask.unsqueeze(1).expand(-1, mask.shape[-1], -1)
    mask = torch.triu(mask)
    return mask

def get_useful_ones(out, label, mask):
    # get mask, mask the padding and down triangle

    mask = mask.reshape(-1)
    tmp_out = out.reshape(-1, out.shape[-1])
    tmp_label = label.reshape(-1)
    # index select, for gpu speed
    indices = mask.nonzero(as_tuple=False).squeeze(-1).long()
    tmp_out = tmp_out.index_select(0, indices)
    tmp_label = tmp_label.index_select(0, indices)

    return tmp_out, tmp_label

def load_model(path):
    checkpoint = torch.load(path, map_location=torch.device('cpu'))
    model = checkpoint['model']
    model.load_state_dict(checkpoint['state_dict'])
    return model




import torch.nn as nn
import torch


class BiaffineLayer(nn.Module):
    def __init__(self, inSize1, inSize2, classSize, dropout=0.3):
        super(BiaffineLayer, self).__init__()

        self.bilinearMap = nn.Parameter(torch.FloatTensor(inSize1 + 1, classSize, inSize2 + 1))
        self.classSize = classSize

    def forward(self, x1, x2):
        # [b, n, v1] -> [b*n, v1]
        # print("BIAFFINEPARA:", self.bilinearMap)
        batch_size = x1.shape[0]
        bucket_size = x1.shape[1]

        x1 = torch.cat((x1,torch.ones([batch_size, bucket_size, 1]).to(x1.device)), axis=2)
        x2 = torch.cat((x2, torch.ones([batch_size, bucket_size, 1]).to(x2.device)), axis=2)
        # Static shape info
        vector_set_1_size = x1.shape[-1]
        vector_set_2_size = x2.shape[-1]

        # [b, n, v1] -> [b*n, v1]
        vector_set_1 = x1.reshape((-1, vector_set_1_size))

        # [v1, r, v2] -> [v1, r*v2]
        bilinear_map = self.bilinearMap.reshape((vector_set_1_size, -1))

        # [b*n, v1] x [v1, r*v2] -> [b*n, r*v2]
        bilinear_mapping = torch.matmul(vector_set_1, bilinear_map)

        # [b*n, r*v2] -> [b, n*r, v2]
        bilinear_mapping = bilinear_mapping.reshape(
            (batch_size, bucket_size * self.classSize, vector_set_2_size))

        # [b, n*r, v2] x [b, n, v2]T -> [b, n*r, n]
        bilinear_mapping = torch.matmul(bilinear_mapping, x2.transpose(1, -1))

        # [b, n*r, n] -> [b, n, r, n]
        bilinear_mapping = bilinear_mapping.reshape(
            (batch_size, bucket_size, self.classSize, bucket_size))

        return bilinear_mapping.transpose(-2, -1)


import torch.nn as nn
import numpy as np
import torch


class FeatureEmbedding(nn.Module):
    """Part-of-Speech embeddings
    Trained with part-of-speech tag, the embedding layer will be the PoS embeddings
    """

    def __init__(self, vocab_size, embedding_size,  dropout=0.1):
        super(FeatureEmbedding, self).__init__()
        self.embedding = nn.Embedding(vocab_size, embedding_size)
        self.embedding.weight.data.copy_ = torch.from_numpy(self.random_embedding(vocab_size, embedding_size))
        self.dropout = nn.Dropout(dropout)

    def random_embedding(self, vocab_size, embedding_dim):
        pretrain_emb = np.empty([vocab_size, embedding_dim])
        scale = np.sqrt(3.0 / embedding_dim)
        for index in range(vocab_size):
            pretrain_emb[index, :] = np.random.uniform(-scale, scale, [1, embedding_dim])
        return pretrain_emb

    def forward(self, x):

        x = self.embedding(x)
        x = self.dropout(x)
        return x



import json

import torch
from torch import nn
from transformers import AutoTokenizer

from vma_nlu.ner.pername_deeplearning.utils import load_model
from vma_nlu.ner.pername_deeplearning.config import Config
from vma_nlu.ner.pername_deeplearning.model.model import Model
# from utils import load_model
# from config import Config

class Inference(object):
    def __init__(self, label_set_path='./vma_nlu/data/label_set.txt', char_vocab_path = './vma_nlu/data/charindex.json', checkpoint = './vma_nlu/data/checkpoint.pth') -> None:
        super().__init__()

        args = Config(label_set_path, char_vocab_path, checkpoint)

        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

        self.tokenizer = AutoTokenizer.from_pretrained(args.model_name_or_path)

        self.model = Model(args)
        print("Loading model .....")
        self.model.load_state_dict(torch.load(args.checkpoint, map_location=torch.device('cpu')))
        # self.model = load_model(args.checkpoint)

        self.model.to(device=self.device)
        self.model.eval()
        

        self.max_seq_len = args.max_seq_len
        self.max_char_len = args.max_char_len

        with open(args.label_set_path, 'r', encoding='utf-8') as f:
            self.label_set = f.read().splitlines()
        self.label_set = {i: w for i, w in enumerate(self.label_set)}

        with open(args.char_vocab_path, 'r', encoding='utf-8') as f:
            self.char_vocab = json.load(f)

        self.softmax = nn.Softmax(dim=2)


    def inference(self, text):
        input_ids, attention_mask, first_subword, char_ids, seq_len = self.preprocessing(text)
        input_ids = input_ids.unsqueeze(0)
        input_ids = input_ids.to(self.device)
        attention_mask = attention_mask.unsqueeze(0)
        attention_mask = attention_mask.to(self.device)
        first_subword = first_subword.unsqueeze(0)
        first_subword = first_subword.to(self.device)
        char_ids = char_ids.unsqueeze(0)
        char_ids = char_ids.to(self.device)
        seq_len = seq_len.unsqueeze(0)
        inputs = {
                'input_ids': input_ids,
                'attention_mask': attention_mask,
                'first_subword': first_subword,
                'char_ids': char_ids,
                'pos_ids': None,
                'fasttext_embs': None
            }
        with torch.no_grad():
            output = self.model(**inputs)
        result = []
        for i in range(len(output)):
            pred = output[i]
            true_len = seq_len[i].item()
            pred = pred[:true_len, :true_len]
            pred = self.prop(pred)
            _, cate_pred = pred.max(dim=-1)
            pred_entity = self.get_entity(cate_pred, pred)
            result.append(pred_entity)
        result = self.output_format(text, result)

        return result

    def preprocessing(self, text):
        # segmentation word using u
        sentence = text.split(' ')
        char_seq = []
        for word in sentence:
            character = self.get_character(word, self.max_char_len)
            char_seq.append(character)
        seq_len = len(sentence)
        seq_len = torch.tensor([seq_len])
        input_ids, attention_mask, firstSWindices = self.preprocess(self.tokenizer, sentence, self.max_seq_len)
        char_ids = self.char2id(char_seq=char_seq, max_seq_len=self.max_seq_len)
        return input_ids, attention_mask, firstSWindices, char_ids, seq_len

    def get_character(self, word, max_char_len):
        word_seq = []
        for i in range(max_char_len): #20
            try:
                char = word[i]
            except:
                char = 'PAD'
            word_seq.append(char)
        return word_seq

    def preprocess(self, tokenizer, sentence, max_seq_len, mask_padding_with_zero=True): # sentence_embedding

        input_ids = [tokenizer.cls_token_id]
        firstSWindices = [len(input_ids)]

        for w in sentence:
            word_token = tokenizer.encode(w)
            input_ids += word_token[1: (len(word_token) - 1)]
            firstSWindices.append(len(input_ids))

        firstSWindices = firstSWindices[: (len(firstSWindices) - 1)]
        input_ids.append(tokenizer.sep_token_id)

        attention_mask = [1 if mask_padding_with_zero else 0] * len(input_ids)

        if len(input_ids) > max_seq_len:
            input_ids = input_ids[:max_seq_len]
            attention_mask = attention_mask[:max_seq_len]
            firstSWindices = firstSWindices[:max_seq_len]
        else:
            attention_mask = attention_mask + [0 if mask_padding_with_zero else 1] * (max_seq_len - len(input_ids))
            input_ids = input_ids + [tokenizer.pad_token_id] * (max_seq_len - len(input_ids))
            firstSWindices = firstSWindices + [0]*(max_seq_len - len(firstSWindices))

        return torch.tensor(input_ids), torch.tensor(attention_mask), torch.tensor(firstSWindices)

    def char2id(self, char_seq, max_seq_len): # char - embedding
        char_ids = []
        for word in char_seq:
            word_char_ids = []
            for char in word:
                if char not in self.char_vocab:
                    word_char_ids.append(self.char_vocab.get("UNK"))
                else:
                    word_char_ids.append(self.char_vocab.get(char))
            char_ids.append(word_char_ids)
        if len(char_ids) < max_seq_len:
            char_ids += [[self.char_vocab.get("PAD")]*self.max_char_len]*(max_seq_len - len(char_ids))
        else:
            char_ids = char_ids[:max_seq_len]
        return torch.tensor(char_ids)

    def get_entity(self, input_tensor, pred):
        entity = []
        for i in range(len(input_tensor)):
            for j in range(i,len(input_tensor)):
                if input_tensor[i][j] > 0:
                    tmp = [i, j, self.label_set[int(input_tensor[i][j])], float(pred[i][j][int(input_tensor[i][j])])]
                    entity.append(tmp)
        return entity
    def prop(self, pred):
        return self.softmax(pred)
    def output_format(self, text, output):
        if not output[0]:
            return {
                'entities': []
            }
        else:
            tmp = []
            tokens = text.split()
            output = output[0]
            for span in output:
                tmp.append({
                    'start': len(" ".join(tokens[:span[0]])),
                    'end': len(" ".join(tokens[:span[1]+1])),
                    'entity': 'person_name',
                    'value': " ".join(tokens[span[0]: span[1]+1]),
                    'confidence': span[3] #
                })
            return {
                'entities': tmp
            }
               
        